//*****************************************************************************
//
// File Name	: 'lcd_lib.c'
// Title		: 8 and 4 _BV LCd interface
// Author		: Scienceprog.com - Copyright (C) 2007
// Created		: 2007-03-29
// Revised		: 2007-08-08
// Version		: 1.0
// Target MCU	: Atmel AVR series
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************
#include "lcd_lib.h"
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>
#include <compat/deprecated.h>

const uint8_t LcdCustomChar[] PROGMEM=//define 8 custom LCD chars
{
	0x00, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x00, // 0. 0/5 full progress block
	0x00, 0x1F, 0x10, 0x10, 0x10, 0x10, 0x1F, 0x00, // 1. 1/5 full progress block
	0x00, 0x1F, 0x18, 0x18, 0x18, 0x18, 0x1F, 0x00, // 2. 2/5 full progress block
	0x00, 0x1F, 0x1C, 0x1C, 0x1C, 0x1C, 0x1F, 0x00, // 3. 3/5 full progress block
	0x00, 0x1F, 0x1E, 0x1E, 0x1E, 0x1E, 0x1F, 0x00, // 4. 4/5 full progress block
	0x00, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x00, // 5. 5/5 full progress block
	0x03, 0x07, 0x0F, 0x1F, 0x0F, 0x07, 0x03, 0x00, // 6. rewind arrow
	0x18, 0x1C, 0x1E, 0x1F, 0x1E, 0x1C, 0x18, 0x00  // 7. fast-forward arrow
};

void setLDP(uint8_t val)
{
	LDP &= ~(_BV(LCD_D7)|_BV(LCD_D6)|_BV(LCD_D5)|_BV(LCD_D4));
	if(val & _BV(7))
		LDP |= _BV(LCD_D7);
	if(val & _BV(6))
		LDP |= _BV(LCD_D6);
	if(val & _BV(5))
		LDP |= _BV(LCD_D5);
	if(val & _BV(4))
		LDP |= _BV(LCD_D4);
}

void LCDsendChar(uint8_t ch)		//Sends Char to LCD
{
	//4 _BV part
	setLDP(ch);
	sbi(LCP,LCD_RS);
	sbi(LCP,LCD_E);	
	_lcd_mini_delay();
	cbi(LCP,LCD_E);
	cbi(LCP,LCD_RS);
	_lcd_mini_delay();

	setLDP(ch<<4);
	sbi(LCP,LCD_RS);
	sbi(LCP,LCD_E);	
	_lcd_mini_delay();
	cbi(LCP,LCD_E);	
	cbi(LCP,LCD_RS);
	_delay_us(50);
}
void LCDsendCommand(uint8_t cmd)	//Sends Command to LCD
{
	//4 _BV part
	setLDP(cmd);
	sbi(LCP,LCD_E);
	_lcd_mini_delay();
	cbi(LCP,LCD_E);
	_lcd_mini_delay();

	setLDP(cmd<<4);
	sbi(LCP,LCD_E);
	_lcd_mini_delay();
	cbi(LCP,LCD_E);

	_delay_us(50);
}
void LCDinit(void)//Initializes LCD
{
	//4 _BV part
	_delay_ms(15);
	//LDP=0x00;
	setLDP(0);
	LCP=0x00;
	LDDR|=_BV(LCD_D7)|_BV(LCD_D6)|_BV(LCD_D5)|_BV(LCD_D4);
	LCDR|=_BV(LCD_E)|_BV(LCD_RW)|_BV(LCD_RS);
   //---------one------
	setLDP(_BV(5)|_BV(4));
	LCP|=_BV(LCD_E);		
	_delay_ms(1);
	LCP&=~(_BV(LCD_E));
	_delay_ms(1);
	//-----------two-----------
	setLDP(_BV(5)|_BV(4));
	LCP|=_BV(LCD_E);		
	_delay_ms(1);
	LCP&=~(_BV(LCD_E));
	_delay_ms(1);
	//-------three-------------
	setLDP(_BV(5));
	LCP|=_BV(LCD_E);		
	_delay_ms(1);
	LCP&=~(_BV(LCD_E));
	_delay_ms(1);
	//--------4 _BV--dual line---------------
	LCDsendCommand(0b00101000);
   //-----increment address, invisible cursor shift------
	LCDsendCommand(0b00001100);
	//init 8 custom chars
	uint8_t ch=0, chn=0;
	while(ch<64)
	{
		LCDdefinechar((LcdCustomChar+ch),chn++);
		ch=ch+8;
	}
}

void LCDstring(uint8_t* data, uint8_t nBytes)	//Outputs string to LCD
{
	register uint8_t i;

	// check to make sure we have a good pointer
	if (!data) return;

	// print data
	for(i=0; i<nBytes; i++)
	{
		if(data[i])
			LCDsendChar(data[i]);
		else
			return;
	}
}
void LCDGotoXY(uint8_t x, uint8_t y)	//Cursor to X Y position
{
	register uint8_t DDRAMAddr;
	// remap lines into proper order
	switch(y)
	{
	case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+x; break;
	case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+x; break;
	case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+x; break;
	case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+x; break;
	default: DDRAMAddr = LCD_LINE0_DDRAMADDR+x;
	}
	// set data address
	LCDsendCommand(1<<LCD_DDRAM | DDRAMAddr);
	
}
//Copies string from flash memory to LCD at x y position
//const uint8_t welcomeln1[] PROGMEM="AVR LCD DEMO\0";
//CopyStringtoLCD(welcomeln1, 3, 1);	
void CopyStringtoLCD(const char *FlashLoc, uint8_t x, uint8_t y)
{
	uint8_t i;
	LCDGotoXY(x,y);
	for(i=0;(uint8_t)pgm_read_byte(&FlashLoc[i]);i++)
	{
		LCDsendChar((uint8_t)pgm_read_byte(&FlashLoc[i]));
	}
}
//defines char symbol in CGRAM
/*
const uint8_t backslash[] PROGMEM= 
{
0b00000000,//back slash
0b00010000,
0b00001000,
0b00000100,
0b00000010,
0b00000001,
0b00000000,
0b00000000
};
LCDdefinechar(backslash,0);
*/
void LCDdefinechar(const uint8_t *pc,uint8_t char_code){
	uint8_t a, pcc;
	uint16_t i;
	a=(char_code<<3)|0x40;
	for (i=0; i<8; i++){
		pcc=pgm_read_byte(&pc[i]);
		LCDsendCommand(a++);
		LCDsendChar(pcc);
		}
}
//******************************
void LCDclr(void)				//Clears LCD
{
	LCDsendCommand(1<<LCD_CLR);
}
void LCDhome(void)			//LCD cursor home
{
	LCDsendCommand(1<<LCD_HOME);
}
void LCDshiftLeft(uint8_t n)	//Scrol n of characters Right
{
	for (uint8_t i=0;i<n;i++)
	{
		LCDsendCommand(0x1E);
	}
}
void LCDshiftRight(uint8_t n)	//Scrol n of characters Left
{
	for (uint8_t i=0;i<n;i++)
	{
		LCDsendCommand(0x18);
	}
}
void LCDcursorOn(void) //displays LCD cursor
{
	LCDsendCommand(0x0E);
}
void LCDcursorOnBlink(void)	//displays LCD blinking cursor
{
	LCDsendCommand(0x0F);
}
void LCDcursorOFF(void)	//turns OFF cursor
{
	LCDsendCommand(0x0C);
}
void LCDblank(void)		//blanks LCD
{
	LCDsendCommand(0x08);
}
void LCDvisible(void)		//Shows LCD
{
	LCDsendCommand(0x0C);
}
void LCDcursorLeft(uint8_t n)	//Moves cursor by n poisitions left
{
	for (uint8_t i=0;i<n;i++)
	{
		LCDsendCommand(0x10);
	}
}
void LCDcursorRight(uint8_t n)	//Moves cursor by n poisitions left
{
	for (uint8_t i=0;i<n;i++)
	{
		LCDsendCommand(0x14);
	}
}
//******************************
//adapted fro mAVRLIB
void LCDprogressBar(uint8_t progress, uint8_t maxprogress, uint8_t length)
{
	uint8_t i;
	uint16_t pixelprogress;
	uint8_t c;

	// draw a progress bar displaying (progress / maxprogress)
	// starting from the current cursor position
	// with a total length of "length" characters
	// ***note, LCD chars 0-5 must be programmed as the bar characters
	// char 0 = empty ... char 5 = full

	// total pixel length of bargraph equals length*PROGRESSPIXELS_PER_CHAR;
	// pixel length of bar itself is
	pixelprogress = ((progress*(length*PROGRESSPIXELS_PER_CHAR))/maxprogress);
	
	// print exactly "length" characters
	for(i=0; i<length; i++)
	{
		// check if this is a full block, or partial or empty
		// (u16) cast is needed to avoid sign comparison warning
		if( ((i*(uint16_t)PROGRESSPIXELS_PER_CHAR)+5) > pixelprogress )
		{
			// this is a partial or empty block
			if( ((i*(uint16_t)PROGRESSPIXELS_PER_CHAR)) > pixelprogress )
			{
				// this is an empty block
				// use space character?
				c = 0;
			}
			else
			{
				// this is a partial block
				c = pixelprogress % PROGRESSPIXELS_PER_CHAR;
			}
		}
		else
		{
			// this is a full block
			c = 5;
		}
		
		// write character to display
		LCDsendChar(c);
	}

}
//******************************
uint8_t int2str(int val, uint8_t *str , char dp, int max)
{
	uint8_t i = 0;
	if(val < 0)
	{
		val = -val;
		str[i] = '-';
		i = 1;
	}

	for(; max ; i++ )
	{
		if(i != dp)
		{
			str[i] = (val/max) + '0';
			if(str[i] > '9')
				str[i] = -'0'+'A';
			val %= max;
			max /= 10;
		}
		else
		{
			str[i] = '.';
		}
	}

	str[i] = 0;
	return i;
}

